import pytest

from gptcomet.utils import is_float


def test_is_float_valid():
    assert is_float("3.14") is True
    assert is_float("123") is True
    assert is_float("-0.123") is True


def test_is_float_invalid():
    assert is_float("abc") is False
    assert is_float("123abc") is False
    assert is_float("abc123") is False


def test_is_float_empty():
    assert is_float("") is False


def test_is_float_non_string():
    with pytest.raises(TypeError):
        is_float(None)



import os
from dataclasses import dataclass
from typing import Any

# Change working dir to tests dir
os.chdir(os.path.dirname(os.path.abspath(__file__)))


@dataclass
class ActiveBranch:
    name: str


@dataclass
class MockGit:
    name: str
    show: Any


@dataclass
class Author:
    name: str
    email: str
    conf_name: str = "name"
    conf_email: str = "email"


@dataclass
class MockRepo:
    active_branch: ActiveBranch
    git: MockGit



import sys
from pathlib import Path
from typing import Annotated, Literal, Optional, cast

import typer
from git import Commit, HookExecutionError, Repo, safe_decode
from httpx import HTTPStatusError
from prompt_toolkit import prompt
from prompt_toolkit.cursor_shapes import CursorShape
from prompt_toolkit.styles import Style
from rich.panel import Panel
from rich.prompt import Prompt

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.const import COMMIT_OUTPUT_TEMPLATE
from gptcomet.exceptions import GitNoStagedChanges, KeyNotFound
from gptcomet.log import set_debug
from gptcomet.message_generator import MessageGenerator
from gptcomet.styles import Colors, stylize
from gptcomet.utils import console

RETRY_INPUT = Literal["y", "n", "r", "e"]


def ask_for_retry() -> RETRY_INPUT:
    """
    Ask the user whether to retry generating a commit message.

    This function is only used in interactive mode.
    Returns:
        Literal["y", "n", "r"]: The user's choice.
    """
    char: RETRY_INPUT
    if sys.stdin.isatty():
        # Interactive mode will ask for confirmation
        char = cast(
            Literal["y", "n", "r", "e"],
            Prompt.ask(
                "Do you want to use this commit message? y: yes, n: no, r: retry, e: edit.",
                default="y",
                choices=["y", "n", "r", "e"],
                case_sensitive=False,
            ),
        )
    else:
        console.print(
            "[yellow]Non-interactive mode detected, using the generated commit message directly.[/yellow]",
        )
        char = "y"

    return char


def edit_text_in_place(initial_message: str) -> str:
    """
    Edit a given text in place with a prompt.

    Args:
        initial_message (str): The initial message to be edited.

    Returns:
        str: The edited message.
    """
    bottom_bar = "Support multiple lines. Type `ESC` and then `Enter` to continue."

    def bottom_toolbar():
        return [('class:bottom-toolbar', f' {bottom_bar} ')]

    style = Style.from_dict({
        'bottom-toolbar': 'fg:#aaaaaa bg:#FDF5E6',
    })

    edited_message = prompt(
        "Edit the message\n",
        default=initial_message,
        multiline=True,
        enable_open_in_editor=True,
        mouse_support=True,
        cursor=CursorShape.BEAM,
        vi_mode=True,
        bottom_toolbar=bottom_toolbar,
        style=style
    )
    console.print(Panel(stylize(edited_message, Colors.GREEN), title="Updated Msg"))
    return edited_message


def gen_output(repo: Repo, commit: Commit, rich=True) -> str:
    """
    Generate a formatted output string for a commit message.

    Args:
        repo (Repo): The git repository object.
        commit (Commit): The git commit object.
        rich (bool, optional): If True, the output will be formatted with rich text markup.
            Defaults to True.

    Returns:
        str: A formatted output string containing information about the commit.
    """
    commit_hash: str = commit.hexsha
    branch: str = repo.active_branch.name
    commit_msg: Optional[str] = safe_decode(commit.message)
    if commit_msg is not None:
        commit_msg = commit_msg.strip()
    no_author = "No Author"
    no_email = "No Email"
    author: Optional[str] = (
        safe_decode(getattr(commit.author, commit.author.conf_name, no_author)) or no_author
    )
    email: Optional[str] = (
        safe_decode(getattr(commit.author, commit.author.conf_email, no_email)) or no_email
    )

    git_show_stat: str = repo.git.show("--pretty=format:", "--stat", commit_hash)

    # Prepare the output format
    if rich is True:
        git_show_stat = (
            git_show_stat.strip().replace("+", "[green]+[/green]").replace("-", "[red]-[/red]")
        )
        author = f":construction_worker: [green]{author}[/]"
        email = f"[blue]{email}[/blue]"
    return COMMIT_OUTPUT_TEMPLATE.format(
        author=author,
        email=email,
        branch=branch,
        commit_hash=commit_hash,
        commit_msg=commit_msg,
        git_show_stat=git_show_stat,
    )


def entry(
    rich: Annotated[bool, typer.Option("--rich", "-r", help="Generate rich commit message.")] = False,
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Print debug information.")] = False,
    local: Annotated[bool, typer.Option("--local", help="Use local configuration file.")] = False,
    config_path: Annotated[
        Optional[Path],
        typer.Option(
            "--config",
            "-c",
            help="Path to config file.",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
    ] = None,
):
    """Generates a commit message based on the staged changes. Will ignore files in `file_ignore`."""
    config_manager = (
        ConfigManager(config_path=config_path) if config_path else get_config_manager(local=local)
    )
    if debug:
        set_debug()

    message_generator = MessageGenerator(config_manager)
    while True:
        try:
            # generate commit message by staged changes
            commit_message = message_generator.generate_commit_message(rich=rich)
        except (KeyNotFound, GitNoStagedChanges, HTTPStatusError) as error:
            console.print(stylize(str(error), Colors.YELLOW))
            raise typer.Exit(0) from None

        if not commit_message:
            console.print(stylize("No commit message generated.", Colors.MAGENTA))
            return

        console.print("Generated commit message:")
        console.print(Panel(stylize(commit_message, Colors.GREEN)))
        try:
            # use generated commit message, retry, edit or discard
            user_input = ask_for_retry()
        except KeyboardInterrupt:
            # catch input interrupt
            return
        if user_input == "y":
            break
        elif user_input == "n":
            console.print(stylize("Commit message discarded.", Colors.YELLOW))
            return
        elif user_input == "e":
            commit_message = edit_text_in_place(commit_message)
            break

    try:
        commit = message_generator.repo.index.commit(message=commit_message)
        output = gen_output(message_generator.repo, commit)
        console.print(output)
    except (HookExecutionError, ValueError) as error:
        console.print(f"Commit Error: {error!s}")
        raise typer.Abort() from None

    console.print(stylize("Commit message saved!", Colors.GREEN))



def test_get_nested_value_dict(config_manager):
    doc = {"a": {"b": {"c": "value"}}}
    keys = "a.b.c"
    assert config_manager.get_nested_value(doc, keys) == "value"


def test_get_nested_value_commented_map(config_manager):
    doc = {"a": {"b": {"c": "value"}}}
    keys = "a.b.c"
    assert config_manager.get_nested_value(doc, keys) == "value"


def test_get_nested_value_not_found(config_manager):
    doc = {"a": {"b": {"c": "value"}}}
    keys = "a.b.d"
    assert config_manager.get_nested_value(doc, keys) is None


def test_get_nested_value_default(config_manager):
    doc = {"a": {"b": {"c": "value"}}}
    keys = "a.b.d"
    default = "default_value"
    assert config_manager.get_nested_value(doc, keys, default) == "default_value"


def test_get_nested_value_keys_list(config_manager):
    doc = {"a": {"b": {"c": "value"}}}
    keys = ["a", "b", "c"]
    assert config_manager.get_nested_value(doc, keys) == "value"


def test_get_nested_value_keys_string(config_manager):
    doc = {"a": {"b": {"c": "value"}}}
    keys = "a.b.c"
    assert config_manager.get_nested_value(doc, keys) == "value"



from unittest.mock import Mock

import pytest

from gptcomet.clis.commit import gen_output
from gptcomet.const import COMMIT_OUTPUT_TEMPLATE


def test_gen_output(repo, commit):
    output = gen_output(repo, commit)
    assert output.startswith(
        COMMIT_OUTPUT_TEMPLATE.format(
            author=":construction_worker: [green]John Doe[/]",
            email="[blue]john@example.com[/blue]",
            branch="master",
            commit_hash="123456",
            commit_msg="test commit message",
            git_show_stat=repo.git.show(),
        )
    )


def test_gen_output_rich_false(repo, commit):
    output = gen_output(repo, commit, rich=False)
    assert output.startswith(
        COMMIT_OUTPUT_TEMPLATE.format(
            author="John Doe",
            email="john@example.com",
            branch="master",
            commit_hash="123456",
            commit_msg="test commit message",
            git_show_stat=repo.git.show(),
        )
    )


def test_gen_output_commit_msg_empty(repo, commit):
    commit.message = ""
    output = gen_output(repo, commit)
    assert output.startswith(
        COMMIT_OUTPUT_TEMPLATE.format(
            author=":construction_worker: [green]John Doe[/]",
            email="[blue]john@example.com[/blue]",
            branch="master",
            commit_hash="123456",
            commit_msg="",
            git_show_stat=repo.git.show(),
        )
    )


def test_gen_output_author_email_empty(repo, commit):
    commit.author.name = None
    commit.author.email = None
    output = gen_output(repo, commit)
    form = COMMIT_OUTPUT_TEMPLATE.format(
        author=":construction_worker: [green]No Author[/]",
        email="[blue]No Email[/blue]",
        branch="master",
        commit_hash="123456",
        commit_msg="test commit message",
        git_show_stat=repo.git.show(),
    )
    assert output == form


def test_gen_output_repo_none():
    with pytest.raises(AttributeError):
        gen_output(None, Mock())


def test_gen_output_commit_none():
    with pytest.raises(AttributeError):
        gen_output(Mock(), None)



from unittest.mock import MagicMock

import pytest

from tests import ActiveBranch, Author, MockGit, MockRepo


@pytest.fixture
def config_manager(tmp_path):
    from gptcomet.config_manager import ConfigManager

    # Mock tmp config file for all fixture
    runtime_config_file = tmp_path / "gptcomet.yaml"
    config_manager = ConfigManager(config_path=runtime_config_file)

    # Mock runtime config file path for CliRunner
    ConfigManager.make_config_path = MagicMock(return_value=config_manager.current_config_path)
    with config_manager.default_config_file.open() as f1, runtime_config_file.open("w") as f2:
        f2.write(f1.read())

    yield config_manager


@pytest.fixture
def repo():
    return MockRepo(
        active_branch=ActiveBranch(name="master"),
        git=MockGit(name="master", show=MagicMock(return_value="git show")),
    )


@pytest.fixture
def commit():
    return MagicMock(
        hexsha="123456",
        message="test commit message",
        author=Author(name="John Doe", email="john@example.com"),
    )



import os
from pathlib import Path
from unittest.mock import MagicMock, create_autospec, patch

import pytest
from git import Git

from gptcomet.message_generator import MessageGenerator


@pytest.fixture
def message_generator():
    os.chdir(Path(__file__).parent.parent.parent)
    config_manager = MagicMock()
    mock_git = create_autospec(Git, instance=True)
    mg = MessageGenerator(config_manager)
    mg.repo.git = mock_git
    return mg


def test_make_ignored_options_empty_ignored_files(message_generator):
    ignored_files = []
    message_generator.repo.git.diff = MagicMock(return_value="")
    assert message_generator.make_ignored_options(ignored_files) == []


def test_make_ignored_options_ignored_files_not_in_staged_files(message_generator):
    ignored_files = ["file1.txt", "file2.txt"]
    message_generator.repo.git.diff = MagicMock(return_value="file3.txt\nfile4.txt")
    with patch("pathlib.Path.exists", return_value=True):
        assert message_generator.make_ignored_options(ignored_files) == []


def test_make_ignored_options_ignored_files_in_staged_files(message_generator):
    ignored_files = ["file1.txt", "file2.txt"]
    message_generator.repo.git.diff = MagicMock(return_value="file1.txt\nfile2.txt\nfile3.txt")
    with patch("pathlib.Path.exists", return_value=True):
        assert message_generator.make_ignored_options(ignored_files) == [
            ":!file1.txt",
            ":!file2.txt",
        ]


def test_make_ignored_options_staged_files_empty(message_generator):
    ignored_files = ["file1.txt", "file2.txt"]
    message_generator.repo.git.diff = MagicMock(return_value="")
    with patch("pathlib.Path.exists", return_value=True):
        assert message_generator.make_ignored_options(ignored_files) == []


def test_make_ignored_options_staged_files_in_ignored_files_pattern(message_generator):
    ignored_files = ["**/pdm.lock", "**/pnpm-lock.yaml"]
    message_generator.repo.git.diff = MagicMock(
        return_value="frontend/pnpm-lock.yaml\nbackend/pdm.lock"
    )
    with patch("pathlib.Path.exists", return_value=True):
        assert message_generator.make_ignored_options(ignored_files) == [
            ":!frontend/pnpm-lock.yaml",
            ":!backend/pdm.lock",
        ]



import asyncio
import fnmatch
import logging
import sys
import typing as t
from functools import wraps

import click
from prompt_toolkit.input import Input, create_input
from prompt_toolkit.key_binding.key_processor import KeyPress
from prompt_toolkit.keys import Keys
from rich import get_console

defenc = sys.getdefaultencoding()
console = get_console()

# context
CONTEXT_SETTINGS = {"help_option_names": ["-h", "--help"]}

LIST_VALUES = ("file_ignore",)

output_language_map = {
    "en": "English",
    "zh-cn": "Simplified Chinese",
    "zh-tw": "Traditional Chinese",
    "fr": "French",
    "vi": "Vietnamese",
    "ja": "Japanese",
    "ko": "Korean",
    "ru": "Russian",
    "tr": "Turkish",
    "id": "Indonesian",
    "th": "Thai",
    "de": "German",
    "es": "Spanish",
}


def convert2type(v: t.Any, _type: t.Optional[type]) -> t.Any:
    if _type is None:
        return v
    if not callable(_type):
        raise TypeError(f"_type must be callable, got {_type}.")  # noqa: TRY003
    try:
        return _type(v)
    except ValueError:
        msg = f"Could not convert value {v} to type {_type}."
        raise ValueError(msg) from None


def strtobool(value: t.Union[str, bool, int]) -> bool:
    """
    Convert a string to a boolean value.

    Parameters:
        value (Union[str, bool, int]): The string to convert.

    Returns:
        bool: The boolean value of the string.

    Raises:
        ValueError: If the string cannot be converted to a boolean value.
    """
    if isinstance(value, bool):
        return value
    elif isinstance(value, int):
        return bool(value)
    true_values = {"ok", "true", "yes", "1", "y", "on"}
    false_values = {"false", "no", "0", "n", "off"}

    value = value.lower()
    if value in true_values:
        return True
    elif value in false_values:
        return False
    else:
        msg = f"{value} is not a valid boolean value, use {true_values | false_values}."
        raise ValueError(msg)


def common_options(func):
    """
    Decorator function that wraps another function with common options and saves them in the context object.

    Parameters:
        func (callable): The function to be wrapped.

    Example:
        >>> @click.command(help="My command")
        >>> @click.pass_context
        >>> @common_options
        >>> def my_command(ctx, debug, local):
        >>>     pass
    Returns:
        callable: The wrapped function.
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        ctx = (
            args[0] if args and isinstance(args[0], click.Context) else click.get_current_context()
        )
        ctx.ensure_object(dict)
        save_common_options(ctx)
        return func(*args, **kwargs)

    wrapper = click.option(
        "--local",
        is_flag=True,
        default=False,
        help="Use local configuration file or global.",
    )(wrapper)
    wrapper = click.option("--debug", is_flag=True, default=False, help="Enable debug mode")(
        wrapper
    )
    return wrapper


def save_common_options(ctx: click.Context):
    """Accept --debug and --local options in any command."""
    if ctx.obj.get("debug") is None or ctx.params["debug"] is True:
        ctx.obj["debug"] = ctx.params["debug"]
    if ctx.obj.get("local") is None or ctx.params["local"] is True:
        ctx.obj["local"] = ctx.params["local"]
    if ctx.obj["debug"] is True:
        logging.getLogger("gptcomet").setLevel(logging.DEBUG)


def is_float(s):
    """
    Checks if a given string can be converted to a float.

    Args:
        s (str): The string to check.

    Returns:
        bool: True if the string can be converted to a float, False otherwise.
    """
    try:
        float(s)
    except ValueError:
        return False
    else:
        return True


def should_ignore(filepath: str, ignore_patterns: list[str]) -> bool:
    """
    Checks if a given filepath should be ignored based on the ignore patterns.

    Args:
        filepath (str): The filepath to check.
        ignore_patterns (list[str]): The list of ignore patterns.

    Returns:
        bool: True if the filepath should be ignored, False otherwise.
    """
    return any(fnmatch.fnmatch(filepath, pattern) for pattern in ignore_patterns)


async def async_raw_input(_stdout=sys.stdout, mask: bool = False, multiline: bool = False) -> str:
    """
    Reads input from the user, with optional character masking, support multiline.
    Support press Esc and then Enter to exit, also support Ctrl+D to exit

    Args:
        _stdout (TextIO, optional): The file to write the input to. Defaults to sys.stdout.
        mask (bool, optional): Whether to mask the input characters. Defaults to False.
        multiline (bool, optional): Whether to allow multiline input. Defaults to False.

    Returns:
        str: The input read from the user.
    """
    _stdout = sys.stdout
    done = asyncio.Event()
    _input: Input = create_input()
    keys = []

    def keys_ready():
        key_press: KeyPress
        pre_key: t.Optional[Keys] = None
        for key_press in _input.read_keys():
            if key_press.key == Keys.Escape:
                pre_key = key_press.key
                continue
            if pre_key == Keys.Escape and key_press.key in (Keys.Enter, Keys.ControlM):
                # press Esc and then Enter to exit
                done.set()
                return
            raw_char = key_press.data.replace("\r", "\n")
            char = "*" if mask else raw_char
            if not multiline and raw_char == "\n":
                done.set()
                return
            keys.append(raw_char)
            _stdout.write(f"{char}")
            _stdout.flush()
            if key_press.key == Keys.ControlD:
                # also support press Ctrl+D to exit
                done.set()
                return
            if key_press.key == Keys.ControlC:
                done.set()
                raise KeyboardInterrupt

    with _input.raw_mode(), _input.attach(keys_ready):
        await done.wait()
    _stdout.write("\n")
    _stdout.flush()
    _input.close()
    return ''.join(keys)


def raw_input(_stdout=sys.stdout, mask: bool = False, multiline: bool = False) -> str:
    """
    Reads input from the user, with optional character masking, synchronously.

    This is a blocking version of `read_input`. It will block until the user presses
    Control-C.

    Args:
        _stdout (TextIO, optional): The file to write the input to. Defaults to sys.stdout.
        mask (bool, optional): Whether to mask the input characters. Defaults to False.
        multiline (bool, optional): Whether to allow multiline input. Defaults to False.

    Returns:
        str: The input read from the user.
    """
    loop = asyncio.new_event_loop()
    return loop.run_until_complete(async_raw_input(_stdout=_stdout, mask=mask, multiline=multiline))



from gptcomet.utils import is_float

KEYS_VALIDATOR = {
    "retries": {"validator": str.isdecimal, "msg": "`retries` must be a positive integer"},
    "max_tokens": {"validator": str.isdecimal, "msg": "`max_tokens` must be a positive integer"},
    "top_p": {
        "validator": lambda x: is_float(x) and 0 <= float(x) <= 1,
        "msg": "`top_p` must be a float in the interval [0, 1]",
    },
    "temperature": {
        "validator": lambda x: is_float(x) and 0.1 <= float(x) <= 1,
        "msg": "`temperature` must be a float in the interval [0.1, 1]",
    },
    "frequency_penalty": {
        "validator": lambda x: is_float(x) and -2 <= float(x) <= 2,
        "msg": "`frequency_penalty` must be a float in the interval [-2, 2]",
    },
}



import logging

from rich.logging import RichHandler

logger = logging.getLogger("gptcomet")

formatter = logging.Formatter(
    "%(message)s: "
)
handler = RichHandler(level=logging.NOTSET, show_path=False)
handler.setFormatter(formatter)
logger.addHandler(handler)

LOG_LEVELS = (
    "CRITICAL",
    "FATAL",
    "ERROR",
    "WARN",
    "WARNING",
    "INFO",
    "DEBUG",
    "NOTSET",
)


def set_level(level):
    if level not in LOG_LEVELS:
        msg = f"Invalid log level: {level}."
        raise ValueError(msg)
    logger.setLevel(level)


def set_debug(debug=True):
    if not debug:
        return
    set_level("DEBUG")



from typing import Annotated

import typer

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.const import GPTCOMET_PRE
from gptcomet.exceptions import ConfigKeyError
from gptcomet.log import logger, set_debug


def entry(
    key: Annotated[str, typer.Argument(..., help="Configuration key to set.")],
    value: Annotated[str, typer.Argument(..., help="Value to set the configuration key to.")],
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Print debug information.")] = False,
    local: Annotated[bool, typer.Option("--local", help="Use local configuration file.")] = False,
):
    cfg: ConfigManager = get_config_manager(local=local)
    if debug:
        set_debug()
        logger.debug(f"Using Config path: {cfg.current_config_path}")
    try:
        cfg.set(key, value)
        styled_key: str = typer.style(key, fg=typer.colors.GREEN)
        styled_value: str = typer.style(value, fg=typer.colors.GREEN)
        typer.echo(f"{GPTCOMET_PRE} Set {styled_key} to {styled_value}.")
    except ConfigKeyError as e:
        typer.echo(f"{GPTCOMET_PRE} Error: {e!s}")



import pytest
from typer.testing import CliRunner

from gptcomet import app

runner = CliRunner()


@pytest.mark.parametrize(
    "command",
    [
        "config get -h",
        "config set -h",
        "config path -h",
        "config keys -h",
        "config list -h",
        "config reset -h",
        "commit -h",
    ],
)
def test_gmsg_commands(command):
    result = runner.invoke(app, command.split())
    assert result.exit_code == 0
    assert "Error" not in result.output



import typing as t
from pathlib import Path

from git import Repo

from gptcomet.config_manager import ConfigManager
from gptcomet.const import FILE_IGNORE_KEY, GPTCOMET_PRE, LANGUAGE_KEY
from gptcomet.exceptions import GitNoStagedChanges
from gptcomet.llm_client import LLMClient
from gptcomet.log import logger
from gptcomet.utils import should_ignore


class MessageGenerator:
    """
    A class that generates concise git messages based on the staged changes.

    Args:
        config_manager (ConfigManager): The gptcomet ConfigManager instance.

    Attributes:
        config_manager (ConfigManager): The ConfigManager instance to use.
        llm_client (LLMClient): The LLMClient instance to use.
        repo (Repo): The Repo instance to use.

    Methods:
        generate_commit_message(rich: bool = True): Generate a commit message from the staged changes.
        make_ignored_options(ignored_files: list[str]): Make a list of ignored files.

    Raises:
        GitNoStagedChanges: If there are no staged changes.
        InvalidGitRepositoryError: If the current directory is not a Git repository.

    Examples:
        >>> from gptcomet.config_manager import ConfigManager
        >>> from gptcomet.message_generator import MessageGenerator
        >>> config_manager = ConfigManager()
        >>> message_generator = MessageGenerator(config_manager)
        >>> message_generator.generate_commit_message()
    """

    __slots__ = ("config_manager", "llm_client", "repo", "diff")

    def __init__(self, config_manager: ConfigManager):
        self.config_manager = config_manager
        self.llm_client = LLMClient(config_manager)
        self.repo = Repo(Path.cwd())
        self.diff = None

    @classmethod
    def from_config_manager(cls, config_manager: ConfigManager):
        """
        Creates an instance of the class from a ConfigManager.

        Args:
            config_manager (ConfigManager): The ConfigManager instance to create the class instance from.

        Returns:
            An instance of the class.
        """
        return cls(config_manager)

    def get_staged_diff(self, repo: t.Optional[Repo] = None) -> str:
        ignored_files: list = self.config_manager.get(FILE_IGNORE_KEY)
        diff_options = ["--staged", "-U2", *self.make_ignored_options(ignored_files)]
        logger.debug(f"{GPTCOMET_PRE} Diff options: {diff_options}")
        repo = repo or self.repo
        diff = repo.git.diff(diff_options)
        lines = diff.splitlines()
        return "\n".join(
            [
                line
                for line in lines
                if not line.startswith("index")
                and not line.startswith("---")
                and not line.startswith("+++")
            ]
        )

    def make_ignored_options(self, ignored_files: list[str]) -> list[str]:
        """
        Make a list of ignored files.

        Args:
            ignored_files (list[str]): The list of ignored files.

        Returns:
            list[str]: The list of ignored files.
        """
        staged_files = self.repo.git.diff(["--staged", "--name-only"]).splitlines()
        return [
            f":!{file}"
            for file in staged_files
            if should_ignore(file, ignored_files) and Path(file).exists()
        ]

    def generate_commit_message(self, rich: bool = False) -> str:
        """
        Generate a commit message from the staged changes.

        Args:
            rich (bool): Whether to use the rich commit message template. Defaults to False.

        Returns:
            str: The generated commit message.

        Raises:
            GitNoStagedChanges: If there are no staged changes.
            BadRequestError: If the completion API returns an error.
        """
        logger.debug(f"{GPTCOMET_PRE} Generating commit message, rich: {rich}")
        diff = self.get_staged_diff()
        logger.debug(f"{GPTCOMET_PRE} Diff length: {len(diff)}")
        self.llm_client.clear_history()
        if not diff:
            raise GitNoStagedChanges()
        return self._translate_msg(self._gen_msg_from_diff(diff, rich=rich))

    def _gen_msg_from_diff(self, diff: str, rich: bool = False) -> str:
        if not rich:
            return self._generate_brief_commit_message(diff)
        else:
            return self._generate_rich_commit_message(diff)

    def _translate_msg(self, msg: str) -> str:
        lang = self.config_manager.get(LANGUAGE_KEY, _type=str)
        title = ""
        if str(lang).lower() != "en":
            if ":" in msg:
                title, msg = msg.split(":")
            # Default is English, but can be changed by the user
            logger.debug(f"{GPTCOMET_PRE} Translating commit message to {lang}")
            translation = str(self.config_manager.get("prompt.translation"))
            translation = translation.replace("{{ placeholder }}", msg).replace(
                "{{ output_language }}", lang
            )
            msg = self.llm_client.generate(translation)
        return f"{title}: {msg}" if title else msg

    def _generate_brief_commit_message(self, diff: str) -> str:
        """
        Generates a brief commit message based on the given diff.

        Args:
            diff (str): The diff string representing the changes made.

        Returns:
            str: The generated brief commit message.

        Raises:
            None

        Description:
            This function uses the "prompt.brief_commit_message" from the config manager to generate a prompt.
            The placeholder "{{ placeholder }}" in the prompt is replaced with the provided diff.

        Example:
            >>> diff = "Add new feature"
            >>> generator = MessageGenerator(ConfigManager())
            >>> commit_message = generator._generate_brief_commit_message(diff)
            >>> print(commit_message)
            Added new feature
        """
        prompt = str(self.config_manager.get("prompt.brief_commit_message"))
        prompt = prompt.replace("{{ placeholder }}", diff)
        return self.llm_client.generate(prompt)

    def _generate_rich_commit_message(self, diff: str) -> str:
        """
        Generates a rich commit message based on the given diff.

        Args:
            diff (str): The diff string representing the changes made.

        Returns:
            str: The generated rich commit message.

        Raises:
            None

        Description:
            This function uses the "prompt.rich_commit_message" and "output.rich_template" from the config manager
            to generate a prompt. The placeholders "{{ rich_template }}" and "{{ placeholder }}" in the prompt
            are replaced with the provided diff and the rich template, respectively.

        Example:
            >>> diff = "Add new feature"
            >>> generator = MessageGenerator(ConfigManager())
            >>> commit_message = generator._generate_rich_commit_message(diff)
            >>> print(commit_message)
            feat: Add new feature

            - Added new feature
        """
        rich_prompt = str(self.config_manager.get("prompt.rich_commit_message"))
        rich_template = str(self.config_manager.get("output.rich_template"))
        rich_prompt = rich_prompt.replace("{{ rich_template }}", rich_template)
        rich_prompt = rich_prompt.replace("{{ placeholder }}", diff)
        return self.llm_client.generate(rich_prompt)



# from click.testing import CliRunner
# from line_profiler import LineProfiler

# from gptcomet import app as cli


# def run_profiling():
#     profiler = LineProfiler()

#     # add target functions
#     profiler.add_function(cli)
#     profiler.add_function(config_set)
#     # start the profiler
#     profiler.enable_by_count()

#     runner = CliRunner()
#     runner.invoke(cli, ["config", "set", "openai.retries", "3"])

#     profiler.disable_by_count()

#     # 打印结果
#     profiler.print_stats()


# if __name__ == "__main__":
#     run_profiling()



from pathlib import Path
from typing import Annotated, Optional

import typer

from gptcomet._types import Provider
from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.const import GPTCOMET_PRE
from gptcomet.log import logger, set_debug
from gptcomet.styles import Colors, stylize
from gptcomet.utils import console, raw_input


def entry(
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Enable debug mode.")] = False,
    local: Annotated[bool, typer.Option("--local", "-l", help="Enable local mode.")] = False,
    config_path: Annotated[
        Optional[Path],
        typer.Option(
            "--config",
            "-c",
            help="Path to config file.",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
    ] = None,
):
    """Setup new provider."""
    cfg: ConfigManager = (
        ConfigManager(config_path=config_path) if config_path else get_config_manager(local=local)
    )
    if debug:
        set_debug()
        logger.debug(f"Using config file: {cfg.current_config_path}")
    provider: str = typer.prompt("Enter provider(lowercase)", default="openai", type=str).lower()
    api_base: str = typer.prompt("Enter API Base", default="https://api.openai.com/v1/")
    model: str = typer.prompt("Enter model", default="text-davinci-003", type=str)

    console.print("Enter API key: ", end="")
    api_key: str = raw_input("Enter API key", mask=True)

    max_tokens: int = typer.prompt("Max tokens", default=1024, type=int)
    provider_cfg: Provider = {
        "api_base": api_base,
        "api_key": api_key,
        "model": model,
        "max_tokens": max_tokens,
        "retries": 2,
    }
    cfg.set_new_provider(provider, provider_cfg)
    console.print(stylize(f"{GPTCOMET_PRE} Provider {provider} set.", Colors.GREEN))



def test_convert2yaml_value_bool(config_manager):
    assert config_manager.convert2yaml_value("true") is True
    assert config_manager.convert2yaml_value("false") is False
    assert config_manager.convert2yaml_value("no") is False
    assert config_manager.convert2yaml_value("0") is False
    assert config_manager.convert2yaml_value("1") is True
    assert config_manager.convert2yaml_value("yes") is True


def test_convert2yaml_value_none(config_manager):
    assert config_manager.convert2yaml_value("none") is None
    assert config_manager.convert2yaml_value("null") is None


def test_convert2yaml_value_float(config_manager):
    assert config_manager.convert2yaml_value("3.14") == 3.14


def test_convert2yaml_value_int(config_manager):
    assert config_manager.convert2yaml_value("42") == 42


def test_convert2yaml_value_str(config_manager):
    assert config_manager.convert2yaml_value("hello") == "hello"


# def test_convert2yaml_value_list(config_manager):
#     assert config_manager.convert2yaml_value("[1, 2, 3]") == [1, 2, 3]



from typing import Annotated

import typer

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.log import logger, set_debug


def entry(
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Print debug information.")] = False,
    local: Annotated[bool, typer.Option("--local", help="Use local configuration file.")] = False,
):
    cfg: ConfigManager = get_config_manager(local=local)
    if debug:
        set_debug()
        logger.debug(f"Using Config path: {cfg.current_config_path}")
    typer.echo(typer.style("Current configuration:\n", fg="green") + cfg.list())



import typer

from gptcomet.clis.config import append, get, keys, path, remove, reset, set
from gptcomet.utils import CONTEXT_SETTINGS

from gptcomet.clis.config import list as _list  # isort:skip

app = typer.Typer(
    name="config",
    no_args_is_help=True,
    context_settings=CONTEXT_SETTINGS,
    short_help="Manage config.",
    help=(
        "Manage gptcomet configuration, default config path is `~/.config/gptcomet/gptcomet.toml`."
    ),
)

app.command(name="get", help="Get config value")(get.entry)
app.command(name="list", help="List config content")(_list.entry)
app.command(name="reset", help="Reset config to default")(reset.entry)
app.command(name="set", help="Set config value")(set.entry)
app.command(name="path", help="Get runtime config path")(path.entry)
app.command(name="keys", help="List supported config keys")(keys.entry)
app.command(
    name="append",
    help="Append a config value to sequence value, not modify if not exists",
    short_help="Append a config value",
)(append.entry)
app.command(
    name="remove",
    help="Remove a config value from sequence value, not modify if not exists",
    short_help="Remove a config value",
)(remove.entry)



import enum

from gptcomet.utils import output_language_map


class GPTCometError(Exception):
    pass


class NotModified(GPTCometError):
    REASON_EXISTS = "Config value already exists and not modified"
    REASON_EMPTY = "Config value is empty"


class KeyNotFound(GPTCometError):
    def __init__(self, key: str):
        self.key = key

    def __str__(self):
        return f"Key '{self.key}' not found in the configuration."


class GitNoStagedChanges(GPTCometError):

    def __str__(self):
        return "No staged changes to commit"


class ConfigErrorEnum(enum.IntEnum):
    """Enum for config error."""

    def __new__(cls, code, description):
        obj = int.__new__(cls, code)
        obj._value_ = code
        obj.description = description
        return obj

    API_KEY_MISSING = 0, "Missing {provider}.api_key in config file"


class ConfigError(GPTCometError):

    def __init__(self, error: ConfigErrorEnum = ConfigErrorEnum.API_KEY_MISSING):
        if not isinstance(error, ConfigErrorEnum):
            raise TypeError
        self.error = error

    def __str__(self):
        return f"Config error: {self.error.description}"


class ConfigKeyError(GPTCometError):
    def __init__(self, key: str):
        self.key = key

    def __str__(self):
        return (
            f"Key '{self.key}' is not allowed. "
            f"Only specified keys can be set, use 'gptcomet config keys' to see the allowed keys."
        )


class ConfigKeyTypeError(GPTCometError):
    def __init__(self, key: str, need_type: str = "list"):
        self.key = key
        self.need_type = need_type

    def __str__(self):
        return f"Key '{self.key}' is not a {self.need_type} type."


class LanguageNotSupportError(GPTCometError):
    def __init__(self, lang: str):
        self.lang = lang
        self.supported = list(output_language_map.keys())

    def __str__(self):
        return f"Language '{self.lang}' not support. Choose from {self.supported}"


class KeyNotSupportError(GPTCometError):
    def __init__(self, key: str):
        self.key = key

    def __str__(self):
        return f"Key '{self.key}' not support."



import logging
import time

logger = logging.getLogger(f"gptcomet.{__name__}")


def timeit_decorator(func):
    def wrapper(*args, **kwargs):
        start_time = time.perf_counter()
        result = func(*args, **kwargs)
        end_time = time.perf_counter()
        logger.debug(f"{func.__name__} executed in {end_time - start_time:.4f} seconds")
        return result
    return wrapper



from typing import Annotated, Union

import typer
from ruamel.yaml.comments import CommentedMap, CommentedSeq

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.exceptions import ConfigKeyError
from gptcomet.log import logger, set_debug


def entry(
    key: Annotated[str, typer.Argument(..., help="The configuration key to get the value for.")],
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Print debug information.")] = False,
    local: Annotated[bool, typer.Option("--local", help="Use local configuration file.")] = False,
):
    cfg: ConfigManager = get_config_manager(local=local)
    if debug:
        set_debug()
        logger.debug(f"Using Config path: {cfg.current_config_path}")
    try:
        value: Union[str, CommentedSeq, CommentedMap] = cfg.get(key)
        styled_key: str = typer.style(key, fg=typer.colors.GREEN)
        typer.echo(f"{styled_key}: {typer.style(value, fg=typer.colors.GREEN)}")
    except ValueError as e:
        typer.echo(f"Error: {e!s}")
    except ConfigKeyError as e:
        typer.echo(str(e))



from decimal import Decimal
from enum import StrEnum

from git import safe_decode
from rich.text import Text

from gptcomet.utils import defenc


class Styles(StrEnum):
    BOLD = "bold"
    ITALIC = "italic"
    UNDERLINE = "underline"
    STRIKETHROUGH = "strikethrough"
    STRIKE = "strike"
    STRIKE_ITALIC = "strike italic"
    STRIKE_BOLD = "strike bold"
    STRIKE_BOLD_ITALIC = "strike bold italic"
    ITALIC_BOLD = "italic bold"
    ITALIC_BOLD_UNDERLINE = "italic bold underline"


class Colors(StrEnum):
    RED = "red"
    GREEN = "green"
    YELLOW = "yellow"
    BLUE = "blue"
    MAGENTA = "magenta"
    CYAN = "cyan"
    WHITE = "white"
    BLACK = "black"
    SKYBLUE = "skyblue"
    SEAGREEN = "seagreen"
    VIOLET = "violet"

    LIGHT_RED_RGB = "rgb(255,100,0)"
    LIGHT_GREEN_RGB = "rgb(173,255,47)"
    LIGHT_YELLOW_RGB = "rgb(255,255,224)"
    LIGHT_BLUE_RGB = "rgb(135,206,235)"
    LIGHT_MAGENTA_RGB = "rgb(255,105,180)"
    LIGHT_CYAN_RGB = "rgb(224,255,255)"
    LIGHT_WHITE_RGB = "rgb(245,245,245)"

    BRIGHT = "bright"

    DEFAULT = "default"


def stylize(text: str, *styles: str) -> Text:
    """
    Format the text with specified styles.

    Args:
        text (str): The text to format.
        *styles (str): The styles to apply to the text.

    Returns:
        str: The formatted text.
    """
    styles = tuple(safe_decode(s) for s in styles)
    if isinstance(text, (int, float, Decimal)):
        text = str(text)
    elif isinstance(text, bytes):
        text = text.decode(defenc)
    return Text(text, style=" ".join(styles))



from typing import Annotated

import typer

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.log import logger, set_debug


def entry(
    debug: Annotated[
        bool, typer.Option("--debug", "-d", help="Print debug information.")
    ] = False,
    local: Annotated[
        bool, typer.Option("--local", help="Use local configuration file.")
    ] = False,
):
    cfg: ConfigManager = get_config_manager(local=local)
    if debug:
        set_debug()
        logger.debug(f"Using Config path: {cfg.current_config_path}")
    cfg.reset()
    typer.echo(f"Configuration `{cfg.current_config_path}` reset to default values")



SUPPORT_KEYS: str = """\
provider                      # LLM provider
file_ignore                   # File to ignore when generating a commit
output.lang                   # Commit message language
console.verbose
{provider}.api_base           # GPT base URL, default openai api
{provider}.api_key
{provider}.model
{provider}.retries
{provider}.proxy
{provider}.max_tokens
{provider}.top_p
{provider}.temperature
{provider}.frequency_penalty
{provider}.extra_headers      # JSON string
prompt.brief_commit_message   # Prompt for brief commit message
prompt.translation            # Prompt for translation commit message to target language
"""



def test_set_nested_value_single_key(config_manager):
    doc = {"a": 0}
    config_manager.set_nested_value(doc, "a", 1)
    assert doc == {"a": 1}


def test_set_nested_value_multiple_keys(config_manager):
    doc = {"a": {"b": {"c": 0}}}
    config_manager.set_nested_value(doc, ["a", "b", "c"], 1)
    assert doc == {"a": {"b": {"c": 1}}}


def test_set_nested_value_create_new_dict(config_manager):
    doc = {"a": {"b": {}}}
    config_manager.set_nested_value(doc, ["a", "b", "c"], 1)
    assert doc == {"a": {"b": {"c": 1}}}



from gptcomet.utils import should_ignore


def test_should_ignore_empty_ignore_patterns():
    filepath = "/path/to/file.txt"
    ignore_patterns = []
    assert should_ignore(filepath, ignore_patterns) is False


def test_should_ignore_not_match():
    filepath = "/path/to/file.txt"
    ignore_patterns = ["*.py"]
    assert should_ignore(filepath, ignore_patterns) is False


def test_should_ignore_match():
    filepath = "/path/to/file.py"
    ignore_patterns = ["*.py"]
    assert should_ignore(filepath, ignore_patterns) is True


def test_should_ignore_multiple_match():
    filepath = "/path/to/file.py"
    ignore_patterns = ["*.py", "*.txt"]
    assert should_ignore(filepath, ignore_patterns) is True



from typing import Annotated

import typer

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.log import logger, set_debug


def entry(
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Print debug information.")] = False,
    local: Annotated[
        bool,
        typer.Option("--local", help="Use local configuration file.", rich_help_panel="Options"),
    ] = False,
):
    cfg: ConfigManager = get_config_manager(local=local)
    if debug:
        set_debug()
        logger.debug(f"Using Config path: {cfg.current_config_path}")
    keys: str = cfg.list_keys()
    typer.echo(typer.style("Supported keys:\n", fg=typer.colors.GREEN) + keys)



from pathlib import Path

import click
from git import Repo

from gptcomet.const import RICH_PREPARE_COMMIT_MSG, SHORT_PREPARE_COMMIT_MSG
from gptcomet.log import logger


class GPTCometHook:
    __slots__ = ("repo", "hook_path")

    def __init__(self):
        """
        Initialize git repository and hook path.

        Raises:
            git.InvalidGitRepositoryError: If the current directory is not a git repository.
            git.NoSuchPathError:
        """
        self.repo = Repo(Path.cwd())
        self.hook_path = Path(self.repo.git_dir) / "hooks" / "prepare-commit-msg"

    def install_hook(self, use_rich: bool = False):
        """
        Install the GPTComet prepare-commit-msg hook.

        Args:
            use_rich (bool): Whether to use the rich commit message template. Defaults to False.
        """
        if use_rich:
            self.hook_path.write_text(RICH_PREPARE_COMMIT_MSG)
        else:
            self.hook_path.write_text(SHORT_PREPARE_COMMIT_MSG)
        self.hook_path.chmod(0o755)

    def is_hook_installed(self):
        """
        Check if the GPTComet hook is installed.
        Returns True if the hook path exists and 'gptcomet' is in the hook path content, False otherwise.
        """
        return self.hook_path.exists() and "gptcomet" in self.hook_path.open().read()

    def uninstall_hook(self):
        """
        Uninstalls the GPTComet prepare-commit-msg hook if it is installed.
        """
        if self.is_hook_installed():
            logger.debug("Uninstalling prepare-commit-msg hook...")
            self.hook_path.unlink()
            click.echo(
                f"[{click.style('GPTComet', fg='green')}] prepare-commit-msg hook has been uninstalled successfully."
            )
        else:
            click.echo(
                f"[{click.style('GPTComet', fg='green')}] prepare-commit-msg hook is not installed."
            )



import typer

from gptcomet import __version__
from gptcomet.clis import commit, config, provider
from gptcomet.utils import CONTEXT_SETTINGS

app = typer.Typer(
    name="gmsg",
    no_args_is_help=True,
    rich_markup_mode="rich",
    context_settings=CONTEXT_SETTINGS,
    pretty_exceptions_enable=False,
)

app.add_typer(config.app, name="config")
app.command("commit")(commit.entry)
app.command("newprovider")(provider.entry)


def version_callback(value: bool):
    if value:
        typer.echo(f"GPTComet Version: {__version__}")
        raise typer.Exit()


@app.callback()
def main(
    version: bool = typer.Option(
        None,
        "-v",
        "--version",
        callback=version_callback,
        is_eager=True,
        help="Show the GPTComet version and exit.",
    ),
):
    pass



from io import StringIO
from pathlib import Path
from typing import Any, Optional, Union

import click
import orjson as json
from glom import glom
from ruamel.yaml import YAML, CommentedMap

from gptcomet._types import CacheType, Provider
from gptcomet.const import LANGUAGE_KEY
from gptcomet.exceptions import (
    ConfigKeyError,
    ConfigKeyTypeError,
    LanguageNotSupportError,
    NotModified,
)
from gptcomet.support_keys import SUPPORT_KEYS
from gptcomet.utils import convert2type, output_language_map, strtobool

yaml = YAML(typ="rt", pure=True)


class ConfigManager:
    __slots__ = (
        "current_config_path",
        "_cache",
        "_valid_keys",
    )
    current_config_path: Path
    _cache: CacheType

    global_config_file: Path = Path.home() / ".config" / "gptcomet" / "gptcomet.yaml"
    default_config_file: Path = Path(__file__).parent / "gptcomet.yaml"

    def __init__(self, config_path: Path):
        # runtime config file
        self.current_config_path = config_path

        self._cache: CacheType = {"config": None, "default_config": None}

    @classmethod
    def from_config_path(cls, config_path: Path) -> "ConfigManager":
        return cls(config_path)

    @property
    def config(self) -> CommentedMap:
        """
        Returns the configuration stored in the cache.

        If the configuration is not already loaded, it is loaded by calling the
        `load_config` method and stored in the cache.

        Returns:
            The configuration stored in the cache.
        """
        if "config" not in self._cache or self._cache["config"] is None:
            self._cache["config"] = self.load_config()
        return self._cache["config"]

    @property
    def default_config(self) -> CommentedMap:
        """
        Returns the default configuration stored in the cache.

        If the default configuration is not already loaded, it is loaded by calling the
        `load_default_config` method and stored in the cache.

        Returns:
            The default configuration stored in the cache.
        """
        if "default_config" not in self._cache or self._cache["default_config"] is None:
            self._cache["default_config"] = self.load_default_config()
        return self._cache["default_config"]

    @classmethod
    def make_config_path(cls, local: bool = False) -> Path:
        return Path.cwd() / ".git" / "gptcomet.yaml" if local else cls.global_config_file

    def is_api_key_set(self) -> bool:
        """
        Checks if the API key is set in the config file.

        Returns:
            bool: True if the API key is set, False if is empty or default.
        """
        if not self.current_config_path.exists():
            return False
        provider = self.get("provider")
        api_key: str = self.get(f"{provider}.api_key")
        return str(api_key).strip("x") != "sk-"

    def get_config_file(self, local: bool = False) -> Path:
        """
        Retrieves the path to the configuration file.

        If local is True, checks if the current directory is a git repository
        and returns the path to .git/gptcomet.yaml if it is, otherwise returns
        the global configuration file path.

        Returns:
            Path: The path to the configuration file.
        """
        config_path = self.global_config_file
        if local:
            cwd = Path.cwd()
            if not (cwd / ".git").exists():
                click.echo(
                    f"[{click.style('GPTComet', fg='yellow')}] Not a git repository. Using global config."
                )
            else:
                config_path = cwd / ".git" / "gptcomet.yaml"
        # click.echo(f"{GPTCOMET_PRE} Using config file: {config_path}")
        return config_path

    def ensure_config_file(self):
        """
        Ensures that the configuration file exists by creating it with default values if it does not already exist.
        """
        config_file = self.current_config_path
        if not config_file.exists():
            config_file.parent.mkdir(parents=True, exist_ok=True)
            with open(self.default_config_file) as default, open(config_file, "w") as target:
                target.write(default.read())
            click.echo(
                f"[{click.style('GPTComet', fg='green')}] Created default config file at {config_file}"
            )

    def load_config(self) -> CommentedMap:
        """
        Load the configuration from the current configuration file.

        Returns:
            CommentedMap: The configuration loaded from the current configuration file.

        """
        self.ensure_config_file()
        with self.current_config_path.open() as f:
            return yaml.load(f)

    def load_default_config(self) -> CommentedMap:
        """
        Load the default configuration from the default configuration file.

        Returns:
            CommentedMap: The default configuration loaded from the file.
        """
        with self.default_config_file.open() as f:
            return yaml.load(f)

    def save_config(self):
        """
        Saves the current configuration to a file.
        """
        with self.current_config_path.open("w") as f:
            yaml.dump(self.config, f)

    def get_nested_value(
        self, doc: Union[dict, CommentedMap], keys: Union[str, list[str]], default: Any = None
    ) -> Any:
        """
        Get the nested value from a dictionary or TOML document using a list of keys.

        Args:
            doc (Union[dict, CommentedMap]): The dictionary or TOML document to retrieve the nested value from.
            keys (Union[str, List[str]]): The list of keys to navigate through the document.
            default (Any, optional): The default value to return if the nested value is not found. Defaults to None.

        Returns:
            Any: The nested value if found, None otherwise.
        """
        if isinstance(keys, list):
            keys = ".".join(keys)
        return glom(doc, keys, default=default)

    def set_nested_value(self, doc: CommentedMap, keys: Union[str, list[str]], value: Any):
        """
        Set a nested value in a dictionary using a list of keys.

        Args:
            doc (TOMLDocument): The dictionary to modify.
            keys (Union[str, List[str]]): The list of keys to navigate through the dictionary.
            value (Any): The value to set at the end of the nested keys.

        It navigates through the dictionary using the keys and sets the value at the end of the nested keys.
        If any key in the path does not exist, it creates a new dictionary at that key.

        Example:
            >>> config = ConfigManager()
            >>> d = {'a': {'b': {'c': 1}}}
            >>> config.set_nested_value(doc, ['a', 'b', 'c'], 2)
            >>> d
            {'a': {'b': {'c': 2}}}

        Note:
            This function modifies the input dictionary `d` in-place.
        """
        if isinstance(keys, str):
            keys = keys.split(".")
        for key in keys[:-1]:
            doc = doc.setdefault(key, {})
        doc[keys[-1]] = value

    def add_provider(self, provider: str):
        if provider in self.config:
            raise ValueError(f"Provider {provider} already exists in config.")  # noqa: TRY003
        info = {
            provider: {
                "model": "xxx",
                "api_key": "xxx",
                "proxy": "",
                "max_tokens": 4096,
                "temperature": 0.7,
                "top_p": 1,
                "frequency_penalty": 0,
                "presence_penalty": 0,
            }
        }
        self.config.update(info)

    def set_new_provider(self, provider: str, provider_info: Provider):
        self.config[provider] = provider_info
        self.save_config()

    def set(self, key: str, value: str):
        """
        Set the value of a configuration key.

        Args:
            key (str): The key to set the value for.
            value (str): The value to set.

        Raises:
            ConfigKeyError: If the key is not supported.

        This method sets the value of a configuration key in the `config` attribute of the `ConfigManager` object.
        It first checks if the key is valid using the `is_valid_key` method. If the key is not valid, it raises a
        `ConfigKeyError` with the invalid key.

        Note:
            This method modifies the toml `config` attribute in-place.
        """
        if key == LANGUAGE_KEY and output_language_map.get(value) is None:
            raise LanguageNotSupportError(key)
        if key.split(".")[-1] not in SUPPORT_KEYS:
            raise ConfigKeyError(key)
        toml_value = self.convert2yaml_value(value)
        self.set_nested_value(self.config, key, toml_value)
        self.save_config()

    def get(self, key: str, default: Optional[Any] = None, _type: Optional[type] = None) -> Any:
        """
        Retrieves the value associated with the given key from the configuration.

        Args:
            key (str): The key to retrieve the value for.
            default (Optional[str]): The default value to return if the key is not found.
            _type (Optional[Type]): The type to convert the retrieved value to.

        Returns:
            Any: The value associated with the given key.

        Raises:
            KeyNotSupportError: If the key is not supported.
            ConfigKeyError: If the key does not exist in the configuration.

        Note:
            This method assumes that the `config` attribute of the `ConfigManager` object is a nested dictionary
            structure with keys separated by dots.
        """
        v = self.get_nested_value(self.config, key, default=default)
        return convert2type(v=v, _type=_type)

    def list(self) -> str:
        """
        Reads the contents of the current configuration file and returns them as a string.

        Returns:
            str: The contents of the current configuration file.

        Raises:
            FileNotFoundError: If the current configuration file does not exist.
        Note:
            This method assumes that `self.current_config_path` is a valid file path.

        """
        self.config.pop("prompt", None)
        t = StringIO()
        yaml.dump(self.config, t)
        text = t.getvalue()
        lines = text.splitlines()
        for i, line in enumerate(lines):
            if "api_key:" in line:
                key = line.split(":")[1].strip(" \"'")
                show_idx = 3
                if key.startswith("sk-or-v1-"):
                    show_idx += 9
                elif key.startswith("sk-"):
                    show_idx += 3
                elif key.startswith("gsk_"):
                    show_idx += 4
                lines[i] = line.replace(key, key[:show_idx] + (len(key) - show_idx) * "*")
        return "\n".join(lines)

    def reset(self):
        """
        Resets the current configuration to its default state.
        """
        with (
            self.default_config_file.open() as default,
            self.current_config_path.open("w") as target,
        ):
            content = default.read()
            target.write(content)
            self._cache["config"] = None

    def list_keys(self) -> str:
        """
        Returns a list of supported keys.
        Returns:
            str: A list of supported keys.
        """
        return SUPPORT_KEYS

    def append(self, key: str, value: str):
        """
        Append a value to a configuration key.

        Args:
            key (str): The key to append the value to.
            value (str): The value to append.

        Raises:
            ConfigKeyError: If the key is not supported.

        This method appends a value to a configuration key in the `config` attribute of the `ConfigManager` object.
        It first checks if the key is valid using the `is_valid_key` method. If the key is not valid, it raises a
        `ConfigKeyError` with the invalid key.

        Note:
            This method modifies the toml `config` attribute in-place.
        """
        current_value = self.get(key)
        if not isinstance(current_value, list):
            raise ConfigKeyTypeError(key, "list")
        if value in current_value:
            raise NotModified(NotModified.REASON_EXISTS)
        current_value.append(value)
        self.save_config()

    def remove(self, key: str, value: str):
        """
        Remove a configuration key.

        Args:
            key (str): The key containing the value to remove.
            value (str): The value to remove.

        Raises:
            ConfigKeyError: If the key is not supported.

        This method removes a configuration key in the `config` attribute of the `ConfigManager` object.
        It first checks if the key is valid. If the key is not valid, it raises a
        `ConfigKeyTypeError` with the invalid key.

        Raises:
            NotModified: If the value is not found in the list.
            ConfigKeyTypeError: If the value is not a list.
            ValueError: If the value not found.

        Note:
            This method modifies the toml `config` attribute in-place.
        """
        current_value = self.get(key)
        if not isinstance(current_value, list):
            raise ConfigKeyTypeError(key)
        if not current_value:
            raise NotModified(NotModified.REASON_EMPTY)
        current_value.remove(value)
        self.save_config()

    def convert2yaml_value(self, value: str) -> Any:
        """
        Converts a string value to a YAML-compatible value.

        Args:
            value (str): The string value to be converted.

        Returns:
            Any: The converted YAML value or the original string value.
        """
        try:
            return strtobool(value)
        except ValueError:
            pass
        if value.lower() in ("none", "null"):
            return None
        if value.isnumeric():
            return int(value)
        try:
            return float(value)
        except ValueError:
            pass
        try:
            # try to parse the value as a literal expression
            return json.loads(value)
        except json.JSONDecodeError:
            pass
        return value


def get_config_manager(local: bool) -> ConfigManager:
    """
    Load the configuration from the current configuration file.

    Args:
        local (bool): Whether to use the local configuration file.

    Returns:
        ConfigManager: The configuration manager with the configuration loaded from the current configuration file.
    """
    return ConfigManager.from_config_path(ConfigManager.make_config_path(local=local))





def test_ensure_configfile_not_exist(config_manager, tmp_path):
    config_manager.current_config_path = tmp_path / "not_exist" / "gptcomet.yaml"
    assert not config_manager.current_config_path.exists()

    config_manager.ensure_config_file()
    assert config_manager.current_config_path.exists()


def test_ensure_configfile_exist(config_manager):
    with config_manager.current_config_path.open() as f:
        before_content = f.read()
    config_manager.ensure_config_file()
    with config_manager.current_config_path.open() as f:
        after_content = f.read()
    assert before_content == after_content



import logging
import typing as t

import click
import httpx
import orjson as json
from glom import glom
from rich.text import Text

from gptcomet.styles import Colors
from gptcomet.utils import console

try:
    import socksio  # noqa: F401
except ImportError as e:
    msg = e.msg
    if "socksio" in msg:
        msg = (
            "Using SOCKS proxy, but the 'socksio' package is not installed. "
            "Make sure to install gptcomet using `pip install gptcomet[socks]` or `pip install socksio`."
        )
    raise ImportError(msg) from None

from gptcomet._types import CompleteParams
from gptcomet.const import (
    DEFAULT_API_BASE,
    DEFAULT_MODEL,
    DEFAULT_RETRIES,
    PROVIDER_KEY,
)
from gptcomet.exceptions import ConfigError, ConfigErrorEnum
from gptcomet.log import logger

if t.TYPE_CHECKING:
    from gptcomet.config_manager import ConfigManager


class LLMClient:
    __slots__ = (
        "config_manager",
        "conversation_history",
        "provider",
        "api_key",
        "model",
        "api_base",
        "retries",
        "completion_path",
        "proxy",
        "content_path",
    )

    @classmethod
    def from_config_manager(cls, config_manager: "ConfigManager"):
        """
        Creates an instance of the class from a ConfigManager.

        Args:
            config_manager (ConfigManager): The ConfigManager instance to create the class instance from.

        Returns:
            An instance of the class.
        """
        return cls(config_manager)

    def __init__(self, config_manager: "ConfigManager"):
        self.config_manager = config_manager
        self.conversation_history: list[dict[str, str]] = []
        self.provider: str = self.config_manager.get(PROVIDER_KEY)
        self.api_key: str = self.config_manager.get(f"{self.provider}.api_key")
        if not self.config_manager.is_api_key_set:
            raise ConfigError(ConfigErrorEnum.API_KEY_MISSING)
        self.model: str = self.config_manager.get(f"{self.provider}.model", DEFAULT_MODEL)
        self.api_base: str = self.config_manager.get(f"{self.provider}.api_base", DEFAULT_API_BASE)
        self.retries: int = int(
            self.config_manager.get(f"{self.provider}.retries", DEFAULT_RETRIES)
        )
        self.completion_path: str = self.config_manager.get(
            f"{self.provider}.completion_path", "/chat/completions"
        )
        self.content_path: str = self.config_manager.get(
            f"{self.provider}.answer_path", "choices.0.message.content"
        )
        self.proxy: str = self.config_manager.get(f"{self.provider}.proxy", "")
        logger.debug(f"Provider: {self.provider}, Model: {self.model}, retries: {self.retries}")

    def generate(self, prompt: str, use_history: bool = False) -> str:
        """
        Generates a response based on the given prompt and conversation history.

        Args:
            prompt (str): The input prompt to generate a response for.
            use_history (bool, optional): Whether to use the conversation history. Defaults to False.

        Returns:
            str: The generated response.

        Raises:
            ConfigError: If the API key is not set in the config.
            BadRequestError: If the completion API returns an error.
        """
        if use_history:
            messages = [*self.conversation_history, {"role": "user", "content": prompt}]
        else:
            messages = [{"role": "user", "content": prompt}]
        params: CompleteParams = self.gen_chat_params(messages)

        # Completion_with_retries returns a dictionary with the response and metadata
        # Could raise BadRequestError error
        response: dict = self.completion_with_retries(**params)
        usage: dict = response.get("usage", {})

        assistant_message: str = glom(response, self.content_path, default="").strip()

        if use_history:
            self.conversation_history.append({"role": "user", "content": prompt})
            self.conversation_history.append({"role": "assistant", "content": assistant_message})
        if not usage:
            console.print("No usage response found.")
        else:
            text = Text("Token usage> prompt tokens: ")
            text.append(f"{usage.get('prompt_tokens')}", Colors.LIGHT_GREEN_RGB)
            text.append(", completion tokens: ")
            text.append(f"{usage.get('completion_tokens')}", Colors.LIGHT_GREEN_RGB)
            text.append(" total tokens: ")
            text.append(f"{usage.get('total_tokens')}", Colors.LIGHT_GREEN_RGB)
            console.print(text)

        return assistant_message

    def gen_chat_params(self, messages: t.Optional[list[dict]] = None) -> CompleteParams:
        """
        Generates the parameters for the chat completion API.

        Returns:
            CompleteParams: The parameters for the chat completion API.
        """
        params: CompleteParams = {
            "model": self.model,
            "api_key": self.api_key,
            "api_base": self.api_base,
            "messages": messages or [],
        }

        # set optional params
        max_tokens = int(self.config_manager.get(f"{self.provider}.max_tokens", 0))
        if max_tokens:
            params["max_tokens"] = max_tokens
        temperature = float(self.config_manager.get(f"{self.provider}.temperature", 0))
        if temperature:
            params["temperature"] = temperature
        top_p = float(self.config_manager.get(f"{self.provider}.top_p", 0.0))
        if top_p:
            params["top_p"] = top_p
        frequency_penalty = float(self.config_manager.get(f"{self.provider}.frequency_penalty", 0))
        if frequency_penalty:
            params["frequency_penalty"] = frequency_penalty
        try:
            extra_headers = json.loads(
                str(self.config_manager.get(f"{self.provider}.extra_headers", "{}"))
            )
        except json.JSONDecodeError:
            click.echo(
                click.style(
                    f"{self.provider}.extra_headers is not a valid JSON string, ignored.",
                    fg="yellow",
                )
            )
            extra_headers = None
        if extra_headers:
            params["extra_headers"] = extra_headers
        return params

    def clear_history(self) -> None:
        """
        Clears the conversation history.
        """
        self.conversation_history = []

    def completion_with_retries(
        self,
        api_base,
        api_key,
        model,
        messages,
        max_tokens,
        temperature=None,
        top_p=None,
        frequency_penalty=None,
        extra_headers=None,
    ) -> dict:
        """
        Wrapper around the completion API that retries on failure.

        Args:
            api_base (str): The base URL for the API.
            api_key (str): The API key to use for authentication.
            model (str): The model to use for completion.
            messages (list[dict]): The messages to send to the API.
            max_tokens (int, optional): The maximum number of tokens to generate. Defaults to None.
            temperature (float, optional): The temperature to use for completion. Defaults to None.
            top_p (float, optional): The top_p to use for completion. Defaults to None.
            frequency_penalty (float, optional): The frequency_penalty to use for completion. Defaults to None.
            extra_headers (dict, optional): Additional headers to send with the request. Defaults to None.

        Returns:
            dict: The response from the API.

        Raises:
            ConfigError: If the API key is not set in the config.
            BadRequestError: If the completion API returns an error.
        """
        transport: httpx.HTTPTransport = httpx.HTTPTransport(retries=self.retries)
        client_params: dict = {"transport": transport}
        if self.proxy:
            client_params["proxies"] = self.proxy
            logger.debug(f"Using proxy: {self.proxy}")
        client: httpx.Client = httpx.Client(**client_params)
        headers = {"Authorization": f"Bearer {api_key}", "Content-Type": "application/json"}

        payload = {
            "model": model,
            "messages": messages,
            "max_tokens": max_tokens,
        }
        if temperature:
            payload["temperature"] = temperature
        if top_p:
            payload["top_p"] = top_p
        if frequency_penalty:
            payload["frequency_penalty"] = frequency_penalty
        if extra_headers:
            headers.update(extra_headers)
        if api_base.endswith("/"):
            api_base = api_base[:-1]
        if not self.completion_path.startswith("/"):
            self.completion_path = "/" + self.completion_path
        url = f"{api_base}{self.completion_path}"

        response: httpx.Response = client.post(
            url,
            json=payload,
            headers=headers,
            timeout=8,
        )
        if logger.level == logging.DEBUG:
            logger.debug(f"completion response: {response.text}")
        response.raise_for_status()
        return response.json()



from typer.testing import CliRunner

from gptcomet import app as cli
from gptcomet.const import GPTCOMET_PRE


def test_config_get(config_manager):
    runner = CliRunner()

    result = runner.invoke(cli, ["config", "get", "provider"])
    assert result.exit_code == 0
    assert result.output.strip() == "provider: " + config_manager.config["provider"]


def test_config_set(config_manager):
    runner = CliRunner()
    result = runner.invoke(cli, ["config", "set", "provider", "openai"])
    assert result.exit_code == 0
    assert result.output.strip() == f"{GPTCOMET_PRE} Set provider to openai."


def test_config_append(config_manager):
    runner = CliRunner()
    result = runner.invoke(cli, ["config", "append", "file_ignore", "README.md"])
    assert result.exit_code == 0
    assert result.output.strip() == f"{GPTCOMET_PRE} Appended README.md to file_ignore."


def test_config_remove_value_error(config_manager):
    runner = CliRunner()
    result = runner.invoke(cli, ["config", "remove", "file_ignore", "README.md"])
    assert result.output.strip() == f"{GPTCOMET_PRE} value not found: README.md"
    assert result.exit_code == 0


def test_config_remove(config_manager):
    runner = CliRunner()
    result = runner.invoke(cli, ["config", "remove", "file_ignore", "yarn.lock"])
    assert result.exit_code == 0
    assert result.output.strip() == "\x1b[1m[\x1b[0mGPTComet\x1b[1m]\x1b[0m Removed yarn.lock from file_ignore."



import os
import sys
from pathlib import Path

import pyinstrument
from click.testing import CliRunner

os.chdir(Path(__file__).resolve().parent.parent.parent)
profiler = pyinstrument.Profiler()
runner = CliRunner()
sys.argv = ["gptcomet", "config", "set", "openai.retries", "n"]
profiler.start()
# ===========================
# entry point of the program
# ===========================
# cmd = ["config", "get", "openai.retries"]
# runner.invoke(cli, cmd, input="n")

# ===========================
# exit program
# ===========================
session = profiler.stop()
# profiler.write_html(Path(__file__).parent / f"pyinstrument_{'-'.join(cmd)}.html", timeline=True)



from rich.text import Text

from gptcomet.styles import stylize


def test_stylize_text():
    text = "Hello, world!"
    styles = ["bold", "italic"]
    result = stylize(text, *styles)
    assert isinstance(result, Text)
    assert result.style == "bold italic"
    assert result._text[0] == "Hello, world!"


def test_stylize_styles():
    text = "Hello, world!"
    styles = ["bold", "italic", "underline"]
    result = stylize(text, *styles)
    assert isinstance(result, Text)
    assert result.style == "bold italic underline"
    assert result._text[0] == "Hello, world!"


def test_stylize_int():
    text = 42
    styles = ["bold", "italic"]
    result = stylize(text, *styles)
    assert isinstance(result, Text)
    assert result.style == "bold italic"
    assert result._text[0] == "42"


def test_stylize_float():
    text = 3.14
    styles = ["bold", "italic"]
    result = stylize(text, *styles)
    assert isinstance(result, Text)
    assert result.style == "bold italic"
    assert result._text[0] == "3.14"


def test_stylize_bytes():
    text = b"Hello, world!"
    styles = ["bold", "italic"]
    result = stylize(text, *styles)
    assert isinstance(result, Text)
    assert result.style == "bold italic"
    assert result._text[0] == "Hello, world!"


def test_stylize_default_style():
    text = "Hello, world!"
    result = stylize(text)
    assert isinstance(result, Text)
    assert result.style == ""
    assert result._text[0] == "Hello, world!"



from typing import Annotated

import typer

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.utils import console


def entry(
    local: Annotated[
        bool,
        typer.Option("--local", help="Use local configuration file.", rich_help_panel="Options"),
    ] = False,
):
    cfg: ConfigManager = get_config_manager(local=local)
    console.print(cfg.current_config_path)



import pytest

from gptcomet.utils import strtobool


def test_true_values():
    true_values = ["ok", "true", "yes", "1", "y", "on"]
    for value in true_values:
        assert strtobool(value) is True


def test_false_values():
    false_values = ["false", "no", "0", "n", "off"]
    for value in false_values:
        assert strtobool(value) is False


def test_invalid_values():
    invalid_values = ["hello", "123", "abc"]
    for value in invalid_values:
        with pytest.raises(ValueError):
            strtobool(value)


def test_case_insensitivity():
    assert strtobool("TRUE") is True
    assert strtobool("FALSE") is False



GPTCOMET_PRE = "[GPTComet]"

DEFAULT_API_BASE = "https://api.openai.com/v1"
DEFAULT_MODEL = "text-davinci-003"
DEFAULT_RETRIES = 2

# GPTComet config keys
LANGUAGE_KEY = "output.lang"
PROVIDER_KEY = "provider"
FILE_IGNORE_KEY = "file_ignore"
CONSOLE_VERBOSE_KEY = "console.verbose"

# git output
COMMIT_OUTPUT_TEMPLATE = """
Auther: {author} <{email}>
{branch}({commit_hash})

{commit_msg}

{git_show_stat}
"""

SHORT_PREPARE_COMMIT_MSG = """\
#!/bin/sh
# GPTComet pre-commit hook

# Run GPTComet commit
gptcomet generate commit

# If GPTComet commit was successful, exit with 0
if [ $? -eq 0 ]; then
    exit 0
else
    echo "GPTComet commit failed. Please review your changes and try again."
    exit 1
fi
"""

RICH_PREPARE_COMMIT_MSG = """\
#!/bin/sh
# GPTComet pre-commit hook

# Run GPTComet commit
gptcomet generate commit --rich

# If GPTComet commit was successful, exit with 0
if [ $? -eq 0 ]; then
    exit 0
else
    echo "GPTComet commit failed. Please review your changes and try again."
    exit 1
fi
"""



from pathlib import Path
from unittest.mock import PropertyMock, patch

import pytest
import yaml

from gptcomet.config_manager import ConfigManager


@pytest.fixture
def mock_config_path(tmp_path):
    return tmp_path / "config.yaml"


@pytest.fixture
def mock_config():
    return {"key": "value"}


def test_save_config_success(mock_config_path, mock_config):
    with (
        patch.object(
            ConfigManager,
            "current_config_path",
            new_callable=PropertyMock,
        ) as mock_current_config_path,
        patch.object(ConfigManager, "config", new_callable=PropertyMock) as mock_config_property,
    ):
        mock_config_property.return_value = mock_config
        mock_current_config_path.return_value = mock_config_path
        config_manager = ConfigManager(config_path=mock_config_path)
        config_manager.save_config()
        assert mock_config_path.exists()
        with mock_config_path.open("r") as f:
            assert yaml.safe_load(f) == mock_config


def test_save_config_fail_path_not_exists(mock_config):
    with (
        patch.object(
            ConfigManager,
            "current_config_path",
            new_callable=PropertyMock,
        ) as mock_current_config_path,
        patch.object(ConfigManager, "config", new_callable=PropertyMock) as mock_config_property,
    ):
        mock_config_property.return_value = mock_config
        mock_current_config_path.return_value = Path("/non/existent/path")
        config_manager = ConfigManager(config_path=Path("/non/existent/path"))
        with pytest.raises(FileNotFoundError):
            config_manager.save_config()


def test_save_config_new_file(mock_config_path, mock_config):
    with (
        patch.object(
            ConfigManager,
            "current_config_path",
            new_callable=PropertyMock,
        ) as mock_current_config_path,
        patch.object(ConfigManager, "config", new_callable=PropertyMock) as mock_config_property,
        patch("builtins.open", side_effect=IOError),
    ):
        mock_config_property.return_value = mock_config
        mock_current_config_path.return_value = mock_config_path
        assert not mock_config_path.exists()
        config_manager = ConfigManager(config_path=mock_config_path)
        config_manager.save_config()
        assert mock_config_path.exists()


def test_save_config_permission_error(mock_config_path, mock_config):
    with (
        patch.object(
            ConfigManager,
            "current_config_path",
            new_callable=PropertyMock,
        ) as mock_current_config_path,
        patch.object(ConfigManager, "config", new_callable=PropertyMock) as mock_config_property,
        patch("io.open", side_effect=PermissionError),
    ):
        mock_config_property.return_value = mock_config
        mock_current_config_path.return_value = mock_config_path
        assert not mock_config_path.exists()
        config_manager = ConfigManager(config_path=mock_config_path)
        with pytest.raises(PermissionError):
            config_manager.save_config()



import typing as t

from ruamel.yaml import CommentedMap

# Config class types
CacheKeys = t.Literal["default_config", "config"]


class CacheType(t.TypedDict):
    default_config: t.Optional[CommentedMap]
    config: t.Optional[CommentedMap]


class CompleteParams(t.TypedDict, total=False):
    """complete required params"""
    api_base: str
    api_key: str
    model: str
    messages: t.Union[list[dict], None, list]
    max_tokens: t.Optional[int]
    temperature: t.Optional[float]
    top_p: t.Optional[float]
    frequency_penalty: t.Optional[float]
    extra_headers: t.Optional[dict]


class Provider(t.TypedDict, total=False):
    """provider setting dict"""
    api_base: str
    api_key: str
    model: str
    max_tokens: int
    temperature: t.Optional[float]
    top_p: t.Optional[float]
    frequency_penalty: t.Optional[float]
    presence_penalty: t.Optional[float]
    extra_headers: t.Optional[str]
    proxy: t.Optional[str]
    retries: t.Optional[int]



import tempfile
from pathlib import Path

from line_profiler import LineProfiler

from gptcomet.config_manager import ConfigManager


def config_set():
    profiler = LineProfiler()
    with tempfile.TemporaryDirectory() as tmpdir:
        config_path = Path(tmpdir) / "getcomment.toml"
        cfg_manager = ConfigManager(config_path=config_path)
        profiler.add_function(cfg_manager.save_config)
        profiler.enable_by_count()
        cfg_manager.set("openai.retries", "2")
        profiler.disable_by_count()
        print("=============config set line profiler==============")
        profiler.print_stats()
        print("=============config set line profiler==============")


def config_keys():
    profiler = LineProfiler()
    with tempfile.TemporaryDirectory() as tmpdir:
        config_path = Path(tmpdir) / "getcomment.toml"
        cfg_manager = ConfigManager(config_path=config_path)
        profiler.add_function(cfg_manager.list_keys)
        profiler.enable_by_count()
        cfg_manager.list_keys()
        profiler.disable_by_count()
        print("=============config keys line profiler==============")
        profiler.print_stats()
        print("=============config keys line profiler==============")


def config_get():
    profiler = LineProfiler()
    with tempfile.TemporaryDirectory() as tmpdir:
        config_path = Path(tmpdir) / "getcomment.toml"
        cfg_manager = ConfigManager(config_path=config_path)
        profiler.add_function(cfg_manager.get)
        profiler.enable_by_count()
        cfg_manager.get("openai.retries")
        profiler.disable_by_count()
        print("=============config get line profiler==============")
        profiler.print_stats()
        print("=============config get line profiler==============")


def config_list():
    profiler = LineProfiler()
    with tempfile.TemporaryDirectory() as tmpdir:
        config_path = Path(tmpdir) / "getcomment.toml"
        cfg_manager = ConfigManager(config_path=config_path)
        profiler.add_function(cfg_manager.list)
        profiler.enable_by_count()
        cfg_manager.ensure_config_file()
        cfg_manager.list()
        profiler.disable_by_count()
        print("=============config list line profiler==============")
        profiler.print_stats()
        print("=============config list line profiler==============")


def config_reset():
    profiler = LineProfiler()
    with tempfile.TemporaryDirectory() as tmpdir:
        config_path = Path(tmpdir) / "getcomment.toml"
        cfg_manager = ConfigManager(config_path=config_path)
        profiler.add_function(cfg_manager.reset)
        profiler.enable_by_count()
        cfg_manager.reset()
        profiler.disable_by_count()
        print("=============config reset line profiler==============")
        profiler.print_stats()
        print("=============config reset line profiler==============")


def config_load_config():
    profiler = LineProfiler()
    with tempfile.TemporaryDirectory() as tmpdir:
        config_path = Path(tmpdir) / "getcomment.toml"
        cfg_manager = ConfigManager(config_path=config_path)
        profiler.add_function(cfg_manager.load_config)
        profiler.enable_by_count()
        cfg_manager.load_config()
        profiler.disable_by_count()
        print("=============config path line profiler==============")
        profiler.print_stats()
        print("=============config path line profiler==============")


def main():
    config_set()
    config_keys()
    config_get()
    config_list()
    config_reset()
    config_load_config()


if __name__ == "__main__":
    main()



from typing import Annotated

import typer

from gptcomet.config_manager import ConfigManager, get_config_manager
from gptcomet.const import GPTCOMET_PRE
from gptcomet.exceptions import ConfigKeyTypeError, NotModified
from gptcomet.log import logger, set_debug
from gptcomet.styles import Colors, stylize
from gptcomet.utils import console


def entry(
    key: Annotated[str, typer.Argument(..., help="Configuration key to set.")],
    value: Annotated[str, typer.Argument(..., help="Value to set the configuration key to.")],
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Print debug information.")] = False,
    local: Annotated[bool, typer.Option("--local", help="Use local configuration file.")] = False,
):
    """Append a value to the list set by the corresponding key."""
    cfg: ConfigManager = get_config_manager(local=local)
    if debug:
        set_debug()
        logger.debug(f"Using Config path: {cfg.current_config_path}")
    try:
        cfg.append(key, value)
        console.print(stylize(f"{GPTCOMET_PRE} Appended {value} to {key}.", Colors.GREEN))

    except NotModified:
        console.print(
            stylize(
                f"{GPTCOMET_PRE} Config value already exists and not modified: {key!s}",
                Colors.LIGHT_BLUE_RGB,
            )
        )
    except ConfigKeyTypeError as e:
        console.print(stylize(f"{GPTCOMET_PRE} Error: {e!s}", Colors.LIGHT_RED_RGB))
